@@ -1,3 +1,4 @@
+from concurrent.futures.process import ProcessPoolExecutor
 from concurrent.futures.thread import ThreadPoolExecutor
 
 import json
@@ -40,7 +41,8 @@ class InternalAutoSolve:
         self.client_key = params["client_key"]
         self.debug = params["debug"]
         self.should_alert_on_cancel = params["should_alert_on_cancel"]
-        self.executor = ThreadPoolExecutor(max_workers=10)
+        self.executor = None
+        self.send_receive_executor = None
         self.queue = []
 
         self.routing_key = ""
@@ -77,6 +79,8 @@ class InternalAutoSolve:
         self.cancel_channel = None
         self.ready = False
         self.connected = False
+        self.executor = ThreadPoolExecutor(max_workers=2)
+        self.send_receive_executor = ThreadPoolExecutor(max_workers=10)
 
         if access_token is not None and api_key is not None:
             self.reset_credentials(access_token, api_key)
@@ -102,12 +106,14 @@ class InternalAutoSolve:
         self.cancel_send_routing_key = self.build_with_access_token(self.autosolve_constants.CANCEL_SEND_ROUTE)
 
     def reset_credentials(self, access_token, api_key):
-        self.close_connection()
+        self.internal_close_connection(False)
         self.access_token = access_token
         self.api_key = api_key
 
     def begin_connection(self):
-        self.executor.submit(self.process_begin_connection)
+        if self.executor.submit(self.process_begin_connection).result():
+            self.executor.submit(self.connect)
+            return True
 
     def process_begin_connection(self):
         sleep_time = self.get_delay(self.delay)
@@ -115,11 +121,11 @@ class InternalAutoSolve:
         if sleep_time > 0:
             time.sleep(sleep_time)
         if self.check_input_values():
-            self.close_connection()
+            self.internal_close_connection(False)
             status_code = self.validate_connection()
             if status_code == 200:
                 self.log("Validation complete", self.autosolve_constants.SUCCESS)
-                self.connect()
+                return True
             elif status_code == 400:
                 self.log(self.autosolve_constants.INVALID_CLIENT_KEY, self.autosolve_constants.ERROR)
                 raise AuthException(self.autosolve_constants.INVALID_CLIENT_KEY)
@@ -156,16 +162,23 @@ class InternalAutoSolve:
         self.log("Beginning message consumption", self.autosolve_constants.SUCCESS)
         self.delay = 0
         self.ready = True
-        self.executor.submit(self.attempt_message_backlog_send)
+        self.send_receive_executor.submit(self.attempt_message_backlog_send)
 
-    def close_connection(self):
+    def internal_close_connection(self, killExecutorService):
+        self.ready = False
         self.log("Closing Connection", self.autosolve_constants.STATUS)
         if self.connection is not None and self.connection.is_open:
             self.connection.close()
             self.connection.ioloop.stop()
+            if killExecutorService:
+                self.executor.shutdown()
+                self.send_receive_executor.shutdown()
         else:
             self.log("Connection does not exist", self.autosolve_constants.STATUS)
 
+    def close_connection(self):
+        self.internal_close_connection(True)
+
     def establish_connection(self, parameters):
         self.log("Attempting connection establishment", self.autosolve_constants.STATUS)
         return pika.SelectConnection(parameters=parameters,
@@ -199,9 +212,9 @@ class InternalAutoSolve:
 
     def receive_message(self, channel, method_frame, header_frame, body):
         if method_frame.routing_key == self.routing_key_receiver:
-            self.on_receiver_message(channel, method_frame, header_frame, body)
+            self.send_receive_executor.submit(self.on_receiver_message, channel, method_frame, header_frame, body)
         else:
-            self.on_cancel_message(channel, method_frame, header_frame, body)
+            self.send_receive_executor.submit(self.on_cancel_message, channel, method_frame, header_frame, body)
 
     def on_receiver_message(self, channel, method_frame, header_frame, body):
         json_string = "".join(chr(x) for x in body)
@@ -221,16 +234,16 @@ class InternalAutoSolve:
         try:
             self.begin_connection()
         except Exception as ex:
-            print(ex)
+            self.log("Exception in reconnect: " + str(ex), self.autosolve_constants.ERROR)
 
     def send_token_request(self, message):
-        self.executor.submit(self.process_send_token_request, message)
+        self.send_receive_executor.submit(self.process_send_token_request, message)
 
     def cancel_token_request(self, taskId):
-        self.executor.submit(self.process_cancel_token_request, taskId)
+        self.send_receive_executor.submit(self.process_cancel_token_request, taskId)
 
     def cancel_all_requests(self):
-        self.executor.submit(self.process_cancel_all_requests)
+        self.send_receive_executor.submit(self.process_cancel_all_requests)
 
     def process_send_token_request(self, message):
         self.log("Sending request message for task: " + str(message['taskId']), self.autosolve_constants.STATUS)
@@ -243,7 +256,7 @@ class InternalAutoSolve:
                          self.autosolve_constants.SUCCESS)
         except Exception as e:
             self.log(e, self.autosolve_constants.WARNING)
-            self.executor.submit(self.resend, message, self.token_send_routing_key, self.directExchangeName)
+            self.send_receive_executor.submit(self.resend, message, self.token_send_routing_key, self.directExchangeName)
 
     def process_cancel_token_request(self, taskId):
         message = dict()
@@ -260,7 +273,7 @@ class InternalAutoSolve:
                          self.autosolve_constants.SUCCESS)
         except Exception as e:
             self.log(e, self.autosolve_constants.WARNING)
-            self.executor.submit(self.resend, message, self.cancel_send_routing_key, self.fanoutExchangeName)
+            self.send_receive_executor.submit(self.resend, message, self.cancel_send_routing_key, self.fanoutExchangeName)
 
     def process_cancel_all_requests(self):
         message = {"apiKey": self.api_key}
@@ -273,7 +286,7 @@ class InternalAutoSolve:
                 self.log("Cancel all requests sent successfully", self.autosolve_constants.SUCCESS)
         except Exception as e:
             self.log(e, self.autosolve_constants.WARNING)
-            self.executor.submit(self.resend, message, self.cancel_send_routing_key, self.fanoutExchangeName)
+            self.send_receive_executor.submit(self.resend, message, self.cancel_send_routing_key, self.fanoutExchangeName)
 
     def send(self):
         req = None
