@@ -1,6 +1,6 @@
 class AutoSolveConstants:
     #NAMES
-    HOSTNAME = "rabbit.autosolve.io"
+    HOSTNAME = "amqp.autosolve.aycd.io"
     VHOST = "oneclick"
     DIRECT_EXCHANGE = "exchanges.direct"
     FANOUT_EXCHANGE = "exchanges.fanout"
@@ -23,6 +23,11 @@ class AutoSolveConstants:
     AUTO_ACK = True
     EXCLUSIVE = False
 
+    #EVENTS
+    ERROR_EVENT = "AutoSolveError"
+    RESPONSE_EVENT = "AutoSolveResponse"
+    CANCEL_RESPONSE_EVENT = "AutoSolveResponse_Cancel"
+
     #ERRORS
     INVALID_CLIENT_KEY = "Invalid Client Key"
     INVALID_API_KEY_OR_ACCESS_TOKEN = "Invalid API or Access Key"
